#!/usr/bin/env python3
import os
from .SOQuery import WebScraper

EXCEPTIONS = [
    'Exception',
    'StopIteration',
    'SystemExit',
    'StandardError',
    'ArithmeticError',
    'OverflowError',
    'FloatingPointError',
    'ZeroDivisionError',
    'AssertionError',
    'AttributeError',
    'EOFError',
    'ImportError',
    'KeyboardInterrupt',
    'LookupError',
    'IndexError',
    'KeyError',
    'NameError',
    'UnboundLocalError',
    'EnvironmentError',
    'IOError',
    'OSError',
    'SyntaxError',
    'IndentationError',
    'SystemError',
    'SystemExit',
    'TypeError',
    'ValueError',
    'RuntimeError',
    'NotImplementedError'
]
   
def main():
    # Ensure that the pipe exists; if not, create it.
    if not os.path.exists(os.environ['HOME'] + '/Desktop/autostack/monitorPipe'):
        os.mkfifo(os.environ['HOME'] + '/Desktop/autostack/monitorPipe')

    # Open the pipe.
    f = open(os.environ['HOME'] + '/Desktop/autostack/monitorPipe', 'r')

    # Inform the user that the script is listening for errors.
    print("Development terminal opened - listening for Python errors...")

    # Listen for new stdout.
    while True:
        try:
            # Read a line from the pipe.
            output = f.readline()

            # Pipe closed.
            if output == '':
                break

            # If it's a python error, scrape SO.
            if output.split()[0][:-1] in EXCEPTIONS:
                # Store user input.
                satisfied = 'no'
                i = 1

                # Find a SO post.
                webscraper = WebScraper()
                searchSoup = webscraper.scrape_so(output.split())

                # Loop over SO post.
                while (satisfied == 'no'):
                    postUrl = webscraper.get_post_url(searchSoup, str(i))
                    
                    if postUrl is None:
                        print("No questions found.")
                    else:
                        answerSoup = webscraper.scrape_question(postUrl)
                        answer_text_soup = webscraper.get_answer(answerSoup)
                        if answer_text_soup is None:
                            print("No answers found.")
                            break
                        else:
                            webscraper.loop_and_print(answer_text_soup)
                    
                    print("Happy with this answer? (yes, no): ")
                    satisfied = input()
                    i += 1

        except UnicodeDecodeError:
            pass

if __name__ == '__main__':
    main()


import setuptools
from os import path

# Long Description.
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'LONGDESCRIPTION.md'), encoding='utf-8') as f:
    long_description = f.read()
    
setuptools.setup(
    # General setup information.
    name='autostack',
    version='0.0.4',
    packages=setuptools.find_packages(exclude=['tests']),
    scripts=[
        'autostack/autostack-terminal'
    ],
    entry_points={
        'console_scripts': [
            'autostack=autostack.__main__:main'
        ]
    },
    install_requires=[
        'Pygments>=2.3.1',
        'requests>=2.21.0',
        'termcolor>=1.1.0',
        'lxml>=4.3.2',
        'beautifulsoup4>=4.7.1',
    ],

    # Meta data.
    author='Elijah Sawyers, Benjamin Sanders, Caleb Werth',
    author_email='elijahsawyers@gmail.com, ben.sanders97@gmail.com, cwerth@crimson.ua.edu',
    description='Automatically detect python errors and search Stack Overflow.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    # TODO: URL.
    # TODO: Keywords.
    url='https://github.com/elijahsawyers/AutoStack',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Debuggers',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
    ],
)


from bs4 import BeautifulSoup
import requests
from termcolor import colored
import pygments
from pygments.lexers import PythonLexer

# Functions to take an error message, search StackOverflow for an answer, and print the answer to the terminal.
class WebScraper:
    # Returns the parsed html of a search page.
    def scrape_so(self, searchStrings):
        # Url string to get html from.
        url = "https://stackoverflow.com/search?q="

        # Build url to search.
        url = url + searchStrings[0]
        for i in searchStrings[1:]:
            url = url + "+" + i

        # Request page.
        r = requests.get(url)

        # Get html from page.
        html_doc = r.text

        # Convert html to BeautfulSoup object.
        soup = BeautifulSoup(html_doc, 'lxml')

        # Return soup object.
        return soup

    # Scrapes html from question.
    def scrape_question(self, url):
        # Builds url to question.
        url = "https://stackoverflow.com/" + url
        print(colored("Getting post from: " + url, 'yellow'))

        # Request page.
        r = requests.get(url)

        # Get html from page.
        html_doc = r.text

        # Convert html to soup object.
        soup = BeautifulSoup(html_doc, 'lxml')

        # Return soup object.
        return soup

    # Gets the url of a post to scrape.
    def get_post_url(self, soup, data_position = "1"):
        # Find first question.
        theDiv = soup.find("div", {"data-position": data_position})

        # If no questions are found.
        if theDiv == None:
            return None
        
        # Find link to page of first question.
        theLink = theDiv.find("a", {"class": "question-hyperlink"})

        # Get link from a tag.
        theRef = theLink.get("href")
        
        # Return link to post.
        return theRef

    # Finds accepted answer, if it exists, and returns a BeautifulSoup object of the html.
    def get_answer(self, soup):
        # Get accepted answer.
        theDiv = soup.find("div", {"itemprop": "acceptedAnswer"})

        # If no accepted answer is found.
        if theDiv is None:
            return None

        # Get text of post.
        theText = theDiv.find("div", {"class": "post-text"})

        # Return post text.
        return theText

    # Traverses through a BeautifulSoup object of the answer's text and prints the text.
    def loop_and_print(self, soup):
        # Formatting in terminal.
        print(colored("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~", 'red'))
        print(colored("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~", 'red'))
        
        # Loop through soup objects.
        for i in soup:
            # If the object is a paragraph.
            if i.name == "p":
                print(colored(i.text, 'white'))

            # If the object is a quote.
            elif i.name == "blockquote":
                print(colored("    " + i.text, 'green'))

            # If the object is in a pre tag.
            elif i.name == "pre":
                # Formatting.
                print("\n")

                # Find the code block.
                code = i.find("code")

                # Store the code.
                code_to_be_colored = ''

                # Loop through code spans.
                for j in code:
                    code_to_be_colored += j

                # Loop over code, and highlight.
                for token, content in pygments.lex(code_to_be_colored, PythonLexer()):
                    if str(token) == "Token.Keyword":
                        print(colored(content, 'blue'), end='')
                    elif str(token) == "Token.Name.Builtin.Pseudo":
                        print(colored(content, 'blue'), end='')
                    elif str(token) == "Token.Literal.Number.Integer":
                        print(colored(content, 'green'), end='')
                    elif str(token) == "Token.Literal.Number.Float":
                        print(colored(content, 'green'), end='')
                    elif str(token) == "Token.Literal.String.Single":
                        print(colored(content, 'yellow'), end='')
                    elif str(token) == "Token.Literal.String.Double":
                        print(colored(content, 'yellow'), end='')
                    elif str(token) == "Token.Literal.String.Doc":
                        print(colored(content, 'yellow'), end='')
                    elif str(token) == "Token.Comment.Single":
                        print(colored(content, 'green'), end='')
                    elif str(token) == "Token.Comment.Hashbang":
                        print(colored(content, 'green'), end='')
                    else:
                        print(content, end='')
                print('')

            # If the object is an unordered list.
            elif i.name == "ul":
                # Loop through list items.
                for li in i.find_all("li"):
                    print(colored("    - " + li.text, 'yellow', attrs=['bold']))
                    # If a link is present.
                    if li.find("a") is not None:
                        # Get link text.
                        link = li.find("a").get("href")
                        print(colored("        link to: " + link, 'yellow', attrs=['bold']))
        print(colored("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~", 'red'))
        print(colored("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~", 'red'))
        return